/***************************************************************************
 *   Copyright (C) 2010 by SIDDHARTH SHARMA siddharth.kde@gmail.com        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>  *
 ***************************************************************************/
#ifndef SKGDATAENGINE_H
#define SKGDATAENGINE_H

#include <Plasma/DataEngine>

#include <skgdocument.h>


/**
 * This class is the skrooge dataengine. It allows fetching data
 * from a skrooge document, by sending SQL orders to the underlying
 * SQLite database
 */
class SKGDataEngine : public Plasma::DataEngine
{
    Q_OBJECT

public:

    /**
     * Base constructor, reimplemented from SKGDataEngine
     * @param iDocument The Skrooge document used to fetch data
     */
    explicit SKGDataEngine(SKGDocument*iDocument, QObject* parent = 0, KService::Ptr service = KService::Ptr(0));

    /**
     * Destructor
     */
    virtual ~SKGDataEngine();

    /**
     * Reimplemented from Plasma::DataEngine
     */
    virtual bool updateSourceEvent(const QString& source);

    /**
     * Reimplemented from Plasma::DataEngine
     */
    virtual bool sourceRequestEvent(const QString& source);

    /**
     * For debug only... Remove me when debug complete"
     */
    void outputData();

protected:

    /**
     * Get some basic data on Accounts
     */
    bool getAccountData();

    /**
     * Get Income and Expenditure Data
     */
    bool getIncomeExpenditureData();

    /**
     * Reimplemented from Plasma::DataEngine
     */
    virtual QStringList sources() const;


private:
    Q_DISABLE_COPY(SKGDataEngine);

    /**
     * The document used to retrieve data
     */
    SKGDocument* document;

};

#endif // SKGDATAENGINE_H
