/***************************************************************************
 *   Copyright (C) 2008 by S. MANKOWSKI / G. DE BURE skrooge@mankowski.fr  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>  *
 ***************************************************************************/
/** @file
 * This file is a test script.
 *
 * @author Stephane MANKOWSKI / Guillaume DE BURE
 */
#include "skgtestmacro.h"
#include "skgimportexportmanager.h"
#include "skgbankincludes.h"

/**
 * The main function of the unit test
 * @param argc the number of arguments
 * @param argv the list of arguments
 */
int main ( int argc, char** argv )
{
    Q_UNUSED ( argc );
    Q_UNUSED ( argv );

    //Init test
    SKGINITTEST ( true );

    {
        //Test import GSK
        SKGDocumentBank document1;
        SKGTESTERROR ( "document1.initialize()", document1.initialize(), true );
        SKGError err;
        {
            //Scope of the transaction
            SKGBEGINTRANSACTION ( document1, "IMPORT_GSB", err );

            SKGImportExportManager imp1 ( &document1, getTestPath ( "IN" ) +"/skgtestimportgsb/test-obfuscated.gsb" );
            SKGTESTERROR ( "GSB.importFile", imp1.importFile(), true );
        }

        {
            SKGAccountObject account ( &document1 );
            SKGTESTERROR ( "GSB.setName", account.setName ( "Account 0" ), true );
            SKGTESTERROR ( "GSB.load", account.load(), true );
            SKGTEST ( "GSB:getCurrentAmount", SKGServices::doubleToString ( account.getCurrentAmount() ), "1029" );
        }
        {
            SKGAccountObject account ( &document1 );
            SKGTESTERROR ( "GSB.setName", account.setName ( "Account 4" ), true );
            SKGTESTERROR ( "GSB.load", account.load(), true );
            SKGTEST ( "GSB:getCurrentAmount", SKGServices::doubleToString ( account.getCurrentAmount() ), "7.5" );
        }
        {
            SKGAccountObject account ( &document1 );
            SKGTESTERROR ( "GSB.setName", account.setName ( "Account 1" ), true );
            SKGTESTERROR ( "GSB.load", account.load(), true );
            SKGTEST ( "GSB:getCurrentAmount", SKGServices::doubleToString ( account.getCurrentAmount() ), "759" );
        }
        {
            SKGAccountObject account ( &document1 );
            SKGTESTERROR ( "GSB.setName", account.setName ( "Account 3" ), true );
            SKGTESTERROR ( "GSB.load", account.load(), true );
            SKGTEST ( "GSB:getCurrentAmount", SKGServices::doubleToString ( account.getCurrentAmount() ), "43.5" );
        }
    }

    {
        //Test import GSK
        SKGDocumentBank document1;
        SKGTESTERROR ( "document1.initialize()", document1.initialize(), true );
        SKGError err;
        {
            //Scope of the transaction
            SKGBEGINTRANSACTION ( document1, "IMPORT_GSB", err );

            SKGImportExportManager imp1 ( &document1, getTestPath ( "IN" ) +"/skgtestimportgsb/version_0.5.9.gsb" );
            SKGTESTERROR ( "GSB.importFile", imp1.importFile(), false );
        }
    }

    {
        //Double import
        SKGDocumentBank document1;
        SKGTESTERROR ( "document1.initialize()", document1.initialize(), true );
        SKGError err;
        {
            //Scope of the transaction
            SKGBEGINTRANSACTION ( document1, "IMPORT_GSB", err );

            SKGImportExportManager imp1 ( &document1, getTestPath ( "IN" ) +"/skgtestimportgsb/essai.gsb" );
            SKGTESTERROR ( "GSB.importFile", imp1.importFile(), true );
        }
        {
            //Scope of the transaction
            SKGBEGINTRANSACTION ( document1, "IMPORT_GSB", err );

            SKGImportExportManager imp1 ( &document1, getTestPath ( "IN" ) +"/skgtestimportgsb/essai.gsb" );
            SKGTESTERROR ( "GSB.importFile", imp1.importFile(), true );
        }
    }
    //End test
    SKGENDTEST();
}
